@@ -72,7 +72,7 @@ class PagedResultsSearchObject:
             else:
               break # no more pages available
 
-      except ldap.SERVER_DOWN,e:
+      except ldap.SERVER_DOWN as e:
         try:
           self.reconnect(self._uri)
         except AttributeError:
@@ -104,4 +104,4 @@ result_pages,all_results = l.paged_search_ext_s(
 
 l.unbind_s()
 
-print 'Received %d results in %d pages.' % (len(all_results),result_pages)
+print('Received %d results in %d pages.' % (len(all_results),result_pages))
