@@ -3,9 +3,6 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See https://www.python-ldap.org/ for details.
 """
-
-from __future__ import unicode_literals
-
 from os import strerror
 
 from ldap.pkginfo import __version__, __author__, __license__
@@ -27,44 +24,20 @@ import warnings
 
 from ldap.schema import SCHEMA_ATTRS
 from ldap.controls import LDAPControl,DecodeControlTuples,RequestControlTuples
-from ldap.extop import ExtendedRequest,ExtendedResponse
-from ldap.compat import reraise
+from ldap.extop import ExtendedRequest,ExtendedResponse,PasswordModifyResponse
 
 from ldap import LDAPError
 
-PY2 = sys.version_info[0] <= 2
-if PY2:
-  text_type = unicode
-else:
-  text_type = str
-
-
-# See SimpleLDAPObject._bytesify_input
-_LDAP_WARN_SKIP_FRAME = True
 
 class LDAPBytesWarning(BytesWarning):
-  """python-ldap bytes mode warning
-  """
+    """Python 2 bytes mode warning"""
 
-def _raise_byteswarning(message):
-    """Raise LDAPBytesWarning
-    """
-
-    # Call stacks that raise the warning tend to be complicated, so
-    # getting a useful stacklevel is tricky.
-    # We walk stack frames, ignoring functions in uninteresting files,
-    # based on the _LDAP_WARN_SKIP_FRAME marker in globals().
-    stacklevel = 2
-    try:
-        getframe = sys._getframe
-    except AttributeError:
-        pass
-    else:
-        frame = sys._getframe(stacklevel)
-        while frame and frame.f_globals.get('_LDAP_WARN_SKIP_FRAME'):
-            stacklevel += 1
-            frame = frame.f_back
-    warnings.warn(message, LDAPBytesWarning, stacklevel=stacklevel+1)
+    def __init__(self, *args, **kwargs):
+        warnings.warn(
+            "LDAPBytesWarning is deprecated and will be removed in the future",
+            DeprecationWarning,
+        )
+        super().__init__(*args, **kwargs)
 
 
 class NO_UNIQUE_ENTRY(ldap.NO_SUCH_OBJECT):
@@ -76,7 +49,9 @@ class NO_UNIQUE_ENTRY(ldap.NO_SUCH_OBJECT):
 
 class SimpleLDAPObject:
   """
-  Drop-in wrapper class around _ldap.LDAPObject
+  This basic class wraps all methods of the underlying C API object.
+
+  The arguments are same as for the :func:`~ldap.initialize()` function.
   """
 
   CLASSATTR_OPTION_MAPPING = {
@@ -94,196 +69,36 @@ class SimpleLDAPObject:
   def __init__(
     self,uri,
     trace_level=0,trace_file=None,trace_stack_limit=5,bytes_mode=None,
-    bytes_strictness=None,
+    bytes_strictness=None, fileno=None
   ):
-    self._trace_level = trace_level
-    self._trace_file = trace_file or sys.stdout
+    self._trace_level = trace_level or ldap._trace_level
+    self._trace_file = trace_file or ldap._trace_file
     self._trace_stack_limit = trace_stack_limit
     self._uri = uri
     self._ldap_object_lock = self._ldap_lock('opcall')
-    self._l = ldap.functions._ldap_function_call(ldap._ldap_module_lock,_ldap.initialize,uri)
+    if fileno is not None:
+      if not hasattr(_ldap, "initialize_fd"):
+        raise ValueError("libldap does not support initialize_fd")
+      if hasattr(fileno, "fileno"):
+        fileno = fileno.fileno()
+      self._l = ldap.functions._ldap_function_call(
+        ldap._ldap_module_lock, _ldap.initialize_fd, fileno, uri
+      )
+    else:
+      self._l = ldap.functions._ldap_function_call(ldap._ldap_module_lock,_ldap.initialize,uri)
     self.timeout = -1
     self.protocol_version = ldap.VERSION3
 
-    # Bytes mode
-    # ----------
+    if bytes_mode:
+        raise ValueError("bytes_mode is *not* supported under Python 3.")
 
-    if PY2:
-        if bytes_mode is None:
-            bytes_mode = True
-            if bytes_strictness is None:
-                _raise_byteswarning(
-                  "Under Python 2, python-ldap uses bytes by default. "
-                  "This will be removed in Python 3 (no bytes for "
-                  "DN/RDN/field names). "
-                  "Please call initialize(..., bytes_mode=False) explicitly.")
-                bytes_strictness = 'warn'
-        else:
-            if bytes_strictness is None:
-                bytes_strictness = 'error'
-    else:
-        if bytes_mode:
-            raise ValueError("bytes_mode is *not* supported under Python 3.")
-        bytes_mode = False
-        bytes_strictness = 'error'
-    self.bytes_mode = bytes_mode
-    self.bytes_strictness = bytes_strictness
+  @property
+  def bytes_mode(self):
+    return False
 
-  def _bytesify_input(self, arg_name, value):
-    """Adapt a value following bytes_mode in Python 2.
-
-    In Python 3, returns the original value unmodified.
-
-    With bytes_mode ON, takes bytes or None and returns bytes or None.
-    With bytes_mode OFF, takes unicode or None and returns bytes or None.
-
-    For the wrong argument type (unicode or bytes, respectively),
-    behavior depends on the bytes_strictness setting.
-    In all cases, bytes or None are returned (or an exception is raised).
-    """
-    if not PY2:
-      return value
-    if value is None:
-      return value
-
-    elif self.bytes_mode:
-      if isinstance(value, bytes):
-        return value
-      elif self.bytes_strictness == 'silent':
-        pass
-      elif self.bytes_strictness == 'warn':
-        _raise_byteswarning(
-            "Received non-bytes value for '{}' in bytes mode; "
-            "please choose an explicit "
-            "option for bytes_mode on your LDAP connection".format(arg_name))
-      else:
-          raise TypeError(
-            "All provided fields *must* be bytes when bytes mode is on; "
-            "got type '{}' for '{}'.".format(type(value).__name__, arg_name)
-          )
-      return value.encode('utf-8')
-    else:
-      if isinstance(value, unicode):
-        return value.encode('utf-8')
-      elif self.bytes_strictness == 'silent':
-        pass
-      elif self.bytes_strictness == 'warn':
-        _raise_byteswarning(
-            "Received non-text value for '{}' with bytes_mode off and "
-            "bytes_strictness='warn'".format(arg_name))
-      else:
-        raise TypeError(
-          "All provided fields *must* be text when bytes mode is off; "
-          "got type '{}' for '{}'.".format(type(value).__name__, arg_name)
-        )
-      return value
-
-  def _bytesify_modlist(self, arg_name, modlist, with_opcode):
-    """Adapt a modlist according to bytes_mode.
-
-    A modlist is a tuple of (op, attr, value), where:
-    - With bytes_mode ON, attr is checked to be bytes
-    - With bytes_mode OFF, attr is converted from unicode to bytes
-    - value is *always* bytes
-    """
-    if not PY2:
-      return modlist
-    if with_opcode:
-      return tuple(
-        (op, self._bytesify_input(arg_name, attr), val)
-        for op, attr, val in modlist
-      )
-    else:
-      return tuple(
-        (self._bytesify_input(arg_name, attr), val)
-        for attr, val in modlist
-      )
-
-  def _unbytesify_text_value(self, value):
-    """Adapt a 'known text, UTF-8 encoded' returned value following bytes_mode.
-
-    With bytes_mode ON, takes bytes or None and returns bytes or None.
-    With bytes_mode OFF, takes bytes or None and returns unicode or None.
-
-    This function should only be applied on field *values*; distinguished names
-    or field *names* are already natively handled in result4.
-    """
-    if value is None:
-      return value
-
-    # Preserve logic of assertions only under Python 2
-    if PY2:
-      assert isinstance(value, bytes), "Expected bytes value, got text instead (%r)" % (value,)
-
-    if self.bytes_mode:
-      return value
-    else:
-      return value.decode('utf-8')
-
-  def _maybe_rebytesify_text(self, value):
-    """Re-encodes text to bytes if needed by bytes_mode.
-
-    Takes unicode (and checks for it), and returns:
-    - bytes under bytes_mode
-    - unicode otherwise.
-    """
-    if not PY2:
-      return value
-
-    if value is None:
-      return value
-
-    assert isinstance(value, text_type), "Should return text, got bytes instead (%r)" % (value,)
-    if not self.bytes_mode:
-      return value
-    else:
-      return value.encode('utf-8')
-
-  def _bytesify_result_value(self, result_value):
-    """Applies bytes_mode to a result value.
-
-    Such a value can either be:
-    - a dict mapping an attribute name to its list of values
-      (where attribute names are unicode and values bytes)
-    - a list of referals (which are unicode)
-    """
-    if not PY2:
-      return result_value
-    if hasattr(result_value, 'items'):
-      # It's a attribute_name: [values] dict
-      return {
-        self._maybe_rebytesify_text(key): value
-        for (key, value) in result_value.items()
-      }
-    elif isinstance(result_value, bytes):
-      return result_value
-    else:
-      # It's a list of referals
-      # Example value:
-      # [u'ldap://DomainDnsZones.xxxx.root.local/DC=DomainDnsZones,DC=xxxx,DC=root,DC=local']
-      return [self._maybe_rebytesify_text(referal) for referal in result_value]
-
-  def _bytesify_results(self, results, with_ctrls=False):
-    """Converts a "results" object according to bytes_mode.
-
-    Takes:
-    - a list of (dn, {field: [values]}) if with_ctrls is False
-    - a list of (dn, {field: [values]}, ctrls) if with_ctrls is True
-
-    And, if bytes_mode is on, converts dn and fields to bytes.
-    """
-    if not PY2:
-      return results
-    if with_ctrls:
-      return [
-        (self._maybe_rebytesify_text(dn), self._bytesify_result_value(fields), ctrls)
-        for (dn, fields, ctrls) in results
-      ]
-    else:
-      return [
-        (self._maybe_rebytesify_text(dn), self._bytesify_result_value(fields))
-        for (dn, fields) in results
-      ]
+  @property
+  def bytes_strictness(self):
+    return 'error'
 
   def _ldap_lock(self,desc=''):
     if ldap.LIBLDAP_R:
@@ -299,7 +114,7 @@ class SimpleLDAPObject:
     self._ldap_object_lock.acquire()
     if __debug__:
       if self._trace_level>=1:
-        self._trace_file.write('*** %s %s - %s\n%s\n' % (
+        self._trace_file.write('*** {} {} - {}\n{}\n'.format(
           repr(self),
           self._uri,
           '.'.join((self.__class__.__name__,func.__name__)),
@@ -317,18 +132,14 @@ class SimpleLDAPObject:
       finally:
         self._ldap_object_lock.release()
     except LDAPError as e:
-      exc_type,exc_value,exc_traceback = sys.exc_info()
       try:
         if 'info' not in e.args[0] and 'errno' in e.args[0]:
           e.args[0]['info'] = strerror(e.args[0]['errno'])
       except IndexError:
         pass
       if __debug__ and self._trace_level>=2:
-        self._trace_file.write('=> LDAPError - %s: %s\n' % (e.__class__.__name__,str(e)))
-      try:
-        reraise(exc_type, exc_value, exc_traceback)
-      finally:
-        exc_type = exc_value = exc_traceback = None
+        self._trace_file.write('=> LDAPError - {}: {}\n'.format(e.__class__.__name__,str(e)))
+      raise
     else:
       if __debug__ and self._trace_level>=2:
         if not diagnostic_message_success is None:
@@ -348,7 +159,7 @@ class SimpleLDAPObject:
     elif name in self.__dict__:
       return self.__dict__[name]
     else:
-      raise AttributeError('%s has no attribute %s' % (
+      raise AttributeError('{} has no attribute {}'.format(
         self.__class__.__name__,repr(name)
       ))
 
@@ -404,9 +215,6 @@ class SimpleLDAPObject:
         The parameter modlist is similar to the one passed to modify(),
         except that no operation integer need be included in the tuples.
     """
-    if PY2:
-        dn = self._bytesify_input('dn', dn)
-        modlist = self._bytesify_modlist('modlist', modlist, with_opcode=False)
     return self._ldap_call(self._l.add_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def add_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
@@ -431,9 +239,6 @@ class SimpleLDAPObject:
     """
     simple_bind([who='' [,cred='']]) -> int
     """
-    if PY2:
-        who = self._bytesify_input('who', who)
-        cred = self._bytesify_input('cred', cred)
     return self._ldap_call(self._l.simple_bind,who,cred,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def simple_bind_s(self,who=None,cred=None,serverctrls=None,clientctrls=None):
@@ -495,14 +300,14 @@ class SimpleLDAPObject:
   def compare_ext(self,dn,attr,value,serverctrls=None,clientctrls=None):
     """
     compare_ext(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int
-    compare_ext_s(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int
+    compare_ext_s(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> bool
     compare(dn, attr, value) -> int
-    compare_s(dn, attr, value) -> int
-        Perform an LDAP comparison between the attribute named attr of
-        entry dn, and the value value. The synchronous form returns 0
-        for false, or 1 for true.  The asynchronous form returns the
-        message id of the initiates request, and the result of the
-        asynchronous compare can be obtained using result().
+    compare_s(dn, attr, value) -> bool
+        Perform an LDAP comparison between the attribute named attr of entry
+        dn, and the value value. The synchronous form returns True or False.
+        The asynchronous form returns the message id of the initiates request,
+        and the result of the asynchronous compare can be obtained using
+        result().
 
         Note that this latter technique yields the answer by raising
         the exception objects COMPARE_TRUE or COMPARE_FALSE.
@@ -510,9 +315,6 @@ class SimpleLDAPObject:
         A design bug in the library prevents value from containing
         nul characters.
     """
-    if PY2:
-        dn = self._bytesify_input('dn', dn)
-        attr = self._bytesify_input('attr', attr)
     return self._ldap_call(self._l.compare_ext,dn,attr,value,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def compare_ext_s(self,dn,attr,value,serverctrls=None,clientctrls=None):
@@ -520,11 +322,11 @@ class SimpleLDAPObject:
     try:
         ldap_res = self.result3(msgid,all=1,timeout=self.timeout)
     except ldap.COMPARE_TRUE:
-      return 1
+      return True
     except ldap.COMPARE_FALSE:
-      return 0
+      return False
     raise ldap.PROTOCOL_ERROR(
-        'Compare operation returned wrong result: %r' % (ldap_res)
+        f'Compare operation returned wrong result: {ldap_res!r}'
     )
 
   def compare(self,dn,attr,value):
@@ -543,7 +345,6 @@ class SimpleLDAPObject:
         form returns the message id of the initiated request, and the
         result can be obtained from a subsequent call to result().
     """
-    dn = self._bytesify_input('dn', dn)
     return self._ldap_call(self._l.delete_ext,dn,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def delete_ext_s(self,dn,serverctrls=None,clientctrls=None):
@@ -583,7 +384,7 @@ class SimpleLDAPObject:
     if extop_resp_class:
       respoid,respvalue = res
       if extop_resp_class.responseName!=respoid:
-        raise ldap.PROTOCOL_ERROR("Wrong OID in extended response! Expected %s, got %s" % (extop_resp_class.responseName,respoid))
+        raise ldap.PROTOCOL_ERROR(f"Wrong OID in extended response! Expected {extop_resp_class.responseName}, got {respoid}")
       return extop_resp_class(extop_resp_class.responseName,respvalue)
     else:
       return res
@@ -592,9 +393,6 @@ class SimpleLDAPObject:
     """
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
     """
-    if PY2:
-        dn = self._bytesify_input('dn', dn)
-        modlist = self._bytesify_modlist('modlist', modlist, with_opcode=True)
     return self._ldap_call(self._l.modify_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def modify_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
@@ -648,15 +446,18 @@ class SimpleLDAPObject:
     return self.rename_s(dn,newrdn,None,delold)
 
   def passwd(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
-    if PY2:
-        user = self._bytesify_input('user', user)
-        oldpw = self._bytesify_input('oldpw', oldpw)
-        newpw = self._bytesify_input('newpw', newpw)
     return self._ldap_call(self._l.passwd,user,oldpw,newpw,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
-  def passwd_s(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
-    msgid = self.passwd(user,oldpw,newpw,serverctrls,clientctrls)
-    return self.extop_result(msgid,all=1,timeout=self.timeout)
+  def passwd_s(self, user, oldpw, newpw, serverctrls=None, clientctrls=None, extract_newpw=False):
+    msgid = self.passwd(user, oldpw, newpw, serverctrls, clientctrls)
+    respoid, respvalue = self.extop_result(msgid, all=1, timeout=self.timeout)
+
+    if respoid != PasswordModifyResponse.responseName:
+      raise ldap.PROTOCOL_ERROR("Unexpected OID %s in extended response!" % respoid)
+    if extract_newpw and respvalue:
+      respvalue = PasswordModifyResponse(PasswordModifyResponse.responseName, respvalue)
+
+    return respoid, respvalue
 
   def rename(self,dn,newrdn,newsuperior=None,delold=1,serverctrls=None,clientctrls=None):
     """
@@ -673,10 +474,6 @@ class SimpleLDAPObject:
         This actually corresponds to the rename* routines in the
         LDAP-EXT C API library.
     """
-    if PY2:
-        dn = self._bytesify_input('dn', dn)
-        newrdn = self._bytesify_input('newrdn', newrdn)
-        newsuperior = self._bytesify_input('newsuperior', newsuperior)
     return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def rename_s(self,dn,newrdn,newsuperior=None,delold=1,serverctrls=None,clientctrls=None):
@@ -765,8 +562,6 @@ class SimpleLDAPObject:
       if add_ctrls:
         resp_data = [ (t,r,DecodeControlTuples(c,resp_ctrl_classes)) for t,r,c in resp_data ]
     decoded_resp_ctrls = DecodeControlTuples(resp_ctrls,resp_ctrl_classes)
-    if resp_data is not None:
-        resp_data = self._bytesify_results(resp_data, with_ctrls=add_ctrls)
     return resp_type, resp_data, resp_msgid, decoded_resp_ctrls, resp_name, resp_value
 
   def search_ext(self,base,scope,filterstr=None,attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
@@ -814,24 +609,8 @@ class SimpleLDAPObject:
         The amount of search results retrieved can be limited with the
         sizelimit parameter if non-zero.
     """
-
-    if PY2:
-        base = self._bytesify_input('base', base)
-        if filterstr is None:
-          # workaround for default argument,
-          # see https://github.com/python-ldap/python-ldap/issues/147
-          if self.bytes_mode:
-            filterstr = b'(objectClass=*)'
-          else:
-            filterstr = u'(objectClass=*)'
-        else:
-          filterstr = self._bytesify_input('filterstr', filterstr)
-        if attrlist is not None:
-            attrlist = tuple(self._bytesify_input('attrlist', a)
-                             for a in attrlist)
-    else:
-      if filterstr is None:
-        filterstr = '(objectClass=*)'
+    if filterstr is None:
+      filterstr = '(objectClass=*)'
     return self._ldap_call(
       self._l.search_ext,
       base,scope,filterstr,
@@ -928,12 +707,8 @@ class SimpleLDAPObject:
 
     Returns: None or text/bytes depending on bytes_mode.
     """
-    if self.bytes_mode:
-      empty_dn = b''
-      attrname = b'subschemaSubentry'
-    else:
-      empty_dn = u''
-      attrname = u'subschemaSubentry'
+    empty_dn = ''
+    attrname = 'subschemaSubentry'
     if dn is None:
       dn = empty_dn
     try:
@@ -956,9 +731,8 @@ class SimpleLDAPObject:
             # If dn was already root DSE we can return here
             return None
         else:
-          # With legacy bytes mode, return bytes; otherwise, since this is a DN,
-          # RFCs impose that the field value *can* be decoded to UTF-8.
-          return self._unbytesify_text_value(search_subschemasubentry_dn)
+          if search_subschemasubentry_dn is not None:
+            return search_subschemasubentry_dn.decode('utf-8')
     except IndexError:
       return None
 
@@ -986,14 +760,9 @@ class SimpleLDAPObject:
     """
     Returns the sub schema sub entry's data
     """
-    if self.bytes_mode:
-      filterstr = b'(objectClass=subschema)'
-      if attrs is None:
-        attrs = [attr.encode('utf-8') for attr in SCHEMA_ATTRS]
-    else:
-      filterstr = u'(objectClass=subschema)'
-      if attrs is None:
-        attrs = SCHEMA_ATTRS
+    filterstr = '(objectClass=subschema)'
+    if attrs is None:
+      attrs = SCHEMA_ATTRS
     try:
       subschemasubentry = self.read_s(
         subschemasubentry_dn,
@@ -1028,12 +797,8 @@ class SimpleLDAPObject:
     """
     convenience wrapper around read_s() for reading rootDSE
     """
-    if self.bytes_mode:
-      base = b''
-      attrlist = attrlist or [b'*', b'+']
-    else:
-      base = u''
-      attrlist = attrlist or [u'*', u'+']
+    base = ''
+    attrlist = attrlist or ['*', '+']
     ldap_rootdse = self.read_s(
       base,
       filterstr=filterstr,
@@ -1046,10 +811,7 @@ class SimpleLDAPObject:
     returns all attribute values of namingContexts in rootDSE
     if namingContexts is not present (not readable) then empty list is returned
     """
-    if self.bytes_mode:
-      name = b'namingContexts'
-    else:
-      name = u'namingContexts'
+    name = 'namingContexts'
     return self.read_rootdse_s(
       attrlist=[name]
     ).get(name, [])
@@ -1057,15 +819,20 @@ class SimpleLDAPObject:
 
 class ReconnectLDAPObject(SimpleLDAPObject):
   """
-  In case of server failure (ldap.SERVER_DOWN) the implementations
-  of all synchronous operation methods (search_s() etc.) are doing
-  an automatic reconnect and rebind and will retry the very same
-  operation.
+  :py:class:`SimpleLDAPObject` subclass whose synchronous request methods
+  automatically reconnect and re-try in case of server failure
+  (:exc:`ldap.SERVER_DOWN`).
 
-  This is very handy for broken LDAP server implementations
-  (e.g. in Lotus Domino) which drop connections very often making
-  it impossible to have a long-lasting control flow in the
-  application.
+  The first arguments are same as for the :py:func:`~ldap.initialize()`
+  function.
+  For automatic reconnects it has additional arguments:
+
+  * retry_max: specifies the number of reconnect attempts before
+    re-raising the :py:exc:`ldap.SERVER_DOWN` exception.
+
+  * retry_delay: specifies the time in seconds between reconnect attempts.
+
+  This class also implements the pickle protocol.
   """
 
   __transient_attrs__ = {
@@ -1079,7 +846,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def __init__(
     self,uri,
     trace_level=0,trace_file=None,trace_stack_limit=5,bytes_mode=None,
-    bytes_strictness=None, retry_max=1, retry_delay=60.0
+    bytes_strictness=None, retry_max=1, retry_delay=60.0, fileno=None
   ):
     """
     Parameters like SimpleLDAPObject.__init__() with these
@@ -1095,7 +862,8 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._last_bind = None
     SimpleLDAPObject.__init__(self, uri, trace_level, trace_file,
                               trace_stack_limit, bytes_mode,
-                              bytes_strictness=bytes_strictness)
+                              bytes_strictness=bytes_strictness,
+                              fileno=fileno)
     self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
     self._retry_max = retry_max
     self._retry_delay = retry_delay
@@ -1123,11 +891,12 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._last_bind = getattr(SimpleLDAPObject, self._last_bind[0]), self._last_bind[1], self._last_bind[2]
     self._ldap_object_lock = self._ldap_lock()
     self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
-    self._trace_file = sys.stdout
-    self.reconnect(self._uri)
+    # XXX cannot pickle file, use default trace file
+    self._trace_file = ldap._trace_file
+    self.reconnect(self._uri,force=True)
 
-  def _store_last_bind(self,method,*args,**kwargs):
-    self._last_bind = (method,args,kwargs)
+  def _store_last_bind(self,_method,*args,**kwargs):
+    self._last_bind = (_method,args,kwargs)
 
   def _apply_last_bind(self):
     if self._last_bind!=None:
@@ -1145,30 +914,39 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def passwd_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.passwd_s,*args,**kwargs)
 
-  def reconnect(self,uri,retry_max=1,retry_delay=60.0):
+  def reconnect(self,uri,retry_max=1,retry_delay=60.0,force=True):
     # Drop and clean up old connection completely
     # Reconnect
     self._reconnect_lock.acquire()
     try:
+      if hasattr(self,'_l'):
+        if force:
+          SimpleLDAPObject.unbind_s(self)
+        else:
+          return
       reconnect_counter = retry_max
       while reconnect_counter:
         counter_text = '%d. (of %d)' % (retry_max-reconnect_counter+1,retry_max)
         if __debug__ and self._trace_level>=1:
-          self._trace_file.write('*** Trying %s reconnect to %s...\n' % (
+          self._trace_file.write('*** Trying {} reconnect to {}...\n'.format(
             counter_text,uri
           ))
         try:
-          # Do the connect
-          self._l = ldap.functions._ldap_function_call(ldap._ldap_module_lock,_ldap.initialize,uri)
-          self._restore_options()
-          # StartTLS extended operation in case this was called before
-          if self._start_tls:
-            SimpleLDAPObject.start_tls_s(self)
-          # Repeat last simple or SASL bind
-          self._apply_last_bind()
+          try:
+            # Do the connect
+            self._l = ldap.functions._ldap_function_call(ldap._ldap_module_lock,_ldap.initialize,uri)
+            self._restore_options()
+            # StartTLS extended operation in case this was called before
+            if self._start_tls:
+              SimpleLDAPObject.start_tls_s(self)
+            # Repeat last simple or SASL bind
+            self._apply_last_bind()
+          except ldap.LDAPError:
+            SimpleLDAPObject.unbind_s(self)
+            raise
         except (ldap.SERVER_DOWN,ldap.TIMEOUT):
           if __debug__ and self._trace_level>=1:
-            self._trace_file.write('*** %s reconnect to %s failed\n' % (
+            self._trace_file.write('*** {} reconnect to {} failed\n'.format(
               counter_text,uri
             ))
           reconnect_counter = reconnect_counter-1
@@ -1177,10 +955,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
           if __debug__ and self._trace_level>=1:
             self._trace_file.write('=> delay %s...\n' % (retry_delay))
           time.sleep(retry_delay)
-          SimpleLDAPObject.unbind_s(self)
         else:
           if __debug__ and self._trace_level>=1:
-            self._trace_file.write('*** %s reconnect to %s successful => repeat last operation\n' % (
+            self._trace_file.write('*** {} reconnect to {} successful => repeat last operation\n'.format(
               counter_text,uri
             ))
           self._reconnects_done = self._reconnects_done + 1
@@ -1190,14 +967,12 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     return # reconnect()
 
   def _apply_method_s(self,func,*args,**kwargs):
-    if not hasattr(self,'_l'):
-      self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay)
+    self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay,force=False)
     try:
       return func(self,*args,**kwargs)
     except ldap.SERVER_DOWN:
-      SimpleLDAPObject.unbind_s(self)
       # Try to reconnect
-      self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay)
+      self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay,force=True)
       # Re-try last operation
       return func(self,*args,**kwargs)
 
