@@ -6,8 +6,6 @@
 #include "berval.h"
 #include "constants.h"
 
-#include "lber.h"
-
 /* Prints to stdout the contents of an array of LDAPControl objects */
 
 /* XXX: This is a debugging tool, and the printf generates some warnings
@@ -82,6 +80,7 @@ Tuple_to_LDAPControl(PyObject *tup)
         return NULL;
 
     lc = PyMem_NEW(LDAPControl, 1);
+
     if (lc == NULL) {
         PyErr_NoMemory();
         return NULL;
@@ -91,6 +90,7 @@ Tuple_to_LDAPControl(PyObject *tup)
 
     len = strlen(oid);
     lc->ldctl_oid = PyMem_NEW(char, len + 1);
+
     if (lc->ldctl_oid == NULL) {
         PyErr_NoMemory();
         LDAPControl_DEL(lc);
@@ -137,6 +137,7 @@ LDAPControls_from_object(PyObject *list, LDAPControl ***controls_ret)
 
     len = PySequence_Length(list);
     ldcs = PyMem_NEW(LDAPControl *, len + 1);
+
     if (ldcs == NULL) {
         PyErr_NoMemory();
         return 0;
@@ -339,6 +340,7 @@ encode_assertion_control(PyObject *self, PyObject *args)
     char *assertion_filterstr;
     struct berval ctrl_val;
     LDAP *ld = NULL;
+    PyThreadState *save;
 
     if (!PyArg_ParseTuple(args, "s:encode_assertion_control",
                           &assertion_filterstr)) {
@@ -348,21 +350,27 @@ encode_assertion_control(PyObject *self, PyObject *args)
     /* XXX: ldap_create() is a nasty and slow hack. It's creating a full blown
      * LDAP object just to encode assertion controls.
      */
-    Py_BEGIN_ALLOW_THREADS err = ldap_create(&ld);
-    Py_END_ALLOW_THREADS if (err != LDAP_SUCCESS)
-        return LDAPerror(ld, "ldap_create");
+    save = PyEval_SaveThread();
+    err = ldap_create(&ld);
+    PyEval_RestoreThread(save);
+    if (err != LDAP_SUCCESS)
+        return LDAPerror(ld);
 
-    err =
-        ldap_create_assertion_control_value(ld, assertion_filterstr,
-                                            &ctrl_val);
+    err = ldap_create_assertion_control_value(ld, assertion_filterstr,
+                                              &ctrl_val);
 
     if (err != LDAP_SUCCESS) {
-        LDAPerror(ld, "ldap_create_assertion_control_value");
-        Py_BEGIN_ALLOW_THREADS ldap_unbind_ext(ld, NULL, NULL);
-        Py_END_ALLOW_THREADS return NULL;
+        LDAPerror(ld);
+        save = PyEval_SaveThread();
+        ldap_unbind_ext(ld, NULL, NULL);
+        PyEval_RestoreThread(save);
+        return NULL;
     }
-    Py_BEGIN_ALLOW_THREADS ldap_unbind_ext(ld, NULL, NULL);
-    Py_END_ALLOW_THREADS res = LDAPberval_to_object(&ctrl_val);
+    save = PyEval_SaveThread();
+    ldap_unbind_ext(ld, NULL, NULL);
+    PyEval_RestoreThread(save);
+    res = LDAPberval_to_object(&ctrl_val);
+
     if (ctrl_val.bv_val != NULL) {
         ber_memfree(ctrl_val.bv_val);
     }
